/**
 * CCCamp07 ScoringSystem
 * A CTF scoring bot & flag+advisory reporting system
 *
 * (C) 2007, Hans-Christian Esperer
 * hc at hcespererorg
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * * Neither the name of the H. Ch. Esperer nor the names of his
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * POSSIBILITY OF SUCH DAMAGE
 **************************************************************************/

package de.sectud.ctf07.scoringsystem;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Map;
import java.util.Random;

import net.sourceforge.adela.interpreter.ADELAFunctionBlock;
import net.sourceforge.adela.interpreter.ADELAInterpreter;
import net.sourceforge.adela.interpreter.ADELAParser;

public class ServiceHandler extends Thread {

	@SuppressWarnings("unused")
	private int sid;

	private String name;

	private String script;

	private ScriptType type;

	private int interval;

	private boolean noQuit = true;

	private Runtime rt = Runtime.getRuntime();

	private Connection connection;

	private Map<String, ADELAFunctionBlock> functions;

	private ADELAFunctionBlock storeFunction = null;

	private ADELAFunctionBlock retrieveFunction = null;

	private ADELAInterpreter evaluator = new ADELAInterpreter();

	public ServiceHandler(Connection connection, int sid, String name,
			String script, ScriptType type, int interval) {
		this.connection = connection;
		this.sid = sid;
		this.name = name;
		this.script = script;
		this.type = type;
		this.interval = interval;

		System.out.println("Created service handler for service " + name);
		start();
	}

	public void quit() {
		this.noQuit = false;
	}

	@Override
	public void run() {
		Random random = new Random();

		byte[] bytes = new byte[32];

		if (type == ScriptType.adela) {
			try {
				ADELAParser parser = new ADELAParser();
				File file = new File("scripts/" + script);
				InputStream reader;
				reader = new FileInputStream(file);
				byte[] script = new byte[(int) file.length()];
				reader.read(script);
				functions = parser.BuildAutomat(new String(script));
				storeFunction = functions.get("store");
				retrieveFunction = functions.get("retrieve");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		while (noQuit) {
			ResultSet mainRS;

			mainRS = null;
			try {
				PreparedStatement mainPS = connection
						.prepareStatement("select flag_name,flag_team,flag_teamhost from flags where flag_collected=false and flag_service=? and flag_disttime<? order by uid");
				mainPS.setString(1, this.name);
				mainPS.setLong(2,
						(Calendar.getInstance().getTimeInMillis() / 1000) - /*
																			 * 5
																			 * minutes
																			 * in
																			 * the
																			 * past
																			 */300);
				mainRS = mainPS.executeQuery();
			} catch (SQLException e1) {
			}

			if (mainRS != null) {
				try {
					while (mainRS.next()) {
						String flagID = mainRS.getString(1);
						String teamID = mainRS.getString(2);
						String teamHost = mainRS.getString(3);
						System.out.println("Trying to get flag " + flagID
								+ " for team " + teamID);
						int retval = -1;
						switch (type) {
						case binary:
							try {
								Process p = rt.exec("scripts/" + script
										+ " retrieve " + teamHost + " "
										+ flagID);
								int max = 0;
								while (max++ < 60) {
									try {
										retval = p.exitValue();
									} catch (IllegalThreadStateException e) {
										Thread.sleep(1000);
										retval = 3;
									}
								}
								if (retval == 3) {
									reportServiceStatus(teamID, this.name,
											retval);
									try {
										p.destroy();
									} catch (Exception e) {
									}
								}
							} catch (IOException e) {
								e.printStackTrace();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;

						case adela:
							try {
								Object[] params = new Object[2];
								params[0] = "IP";
								params[1] = flagID;
								retval = (Integer) retrieveFunction.Handle(
										evaluator, params);
							} catch (Exception e) {
								e.printStackTrace();
							}
							break;
						}
						PreparedStatement ps;
						ps = connection
								.prepareStatement("update flags set flag_collected=true where flag_name=?");
						ps.setString(1, flagID);
						ps.execute();
						ps = connection
								.prepareStatement("update flags set flag_collectingteam=? where flag_name=?");
						if (retval == 0) {
							PreparedStatement ps2 = connection
									.prepareStatement("select team_points_defensive from teams where team_name=?");
							ps2.setString(1, teamID);
							ResultSet rs2 = ps2.executeQuery();
							rs2.next();
							int points = rs2.getInt(1);
							PreparedStatement ps3 = connection
									.prepareStatement("update teams set team_points_defensive=? where team_name=?");
							ps3.setInt(1, points + 1);
							ps3.setString(2, teamID);
							ps3.execute();
							ps.setString(1, teamID);
						} else {
							ps.setString(1, "none");
						}
						ps.setString(2, flagID);
						ps.execute();

						reportServiceStatus(teamID, this.name, retval);
					}
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}

			mainRS = null;
			try {
				PreparedStatement mainPS = connection
						.prepareStatement("select uid,team_name,team_host from teams");
				mainRS = mainPS.executeQuery();
			} catch (SQLException e1) {
			}

			if (mainRS != null) {

				try {
					while (mainRS.next()) {
						String teamName = mainRS.getString(2);
						String teamHost = mainRS.getString(3);

						random.nextBytes(bytes);
						String flagID = Stuff.toHex(bytes);

						System.out.println("Checking service " + this.name
								+ " for team " + teamName);
						int retval = -1;
						switch (type) {
						case binary:
							try {
								Process p = rt.exec("scripts/" + script
										+ " store " + teamHost + " " + flagID);
								int max = 0;
								while (max++ < 60) {
									try {
										retval = p.exitValue();
									} catch (IllegalThreadStateException e) {
										Thread.sleep(1000);
										retval = 3;
									}
								}
								if (retval == 3) {
									reportServiceStatus(teamName, this.name,
											retval);
									try {
										p.destroy();
									} catch (Exception e) {
									}
								}
							} catch (IOException e) {
								e.printStackTrace();
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
							break;

						case adela:
							try {
								Object[] params = new Object[2];
								params[0] = "IP";
								params[1] = flagID;
								retval = (Integer) storeFunction.Handle(
										evaluator, params);
							} catch (Exception e) {
								e.printStackTrace();
							}
							break;
						}
						if (retval != 0) {
							reportServiceStatus(teamName, this.name, 2);
							System.out.println("Service " + this.name
									+ " not up");
						} else {
							// store flag in database
							PreparedStatement ps = connection
									.prepareStatement("insert into flags(flag_name,flag_collected,flag_team,flag_service,flag_teamhost,flag_disttime) values(?,?,?,?,?,?)");
							ps.setString(1, flagID);
							ps.setBoolean(2, false);
							ps.setString(3, teamName);
							ps.setString(4, this.name);
							ps.setString(5, teamHost);
							ps.setLong(6, Calendar.getInstance()
									.getTimeInMillis() / 1000);
							ps.execute();
						}
					}
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			try {
				sleep(this.interval * 1000);
			} catch (InterruptedException e) {
			}
		}
	}

	private void reportServiceStatus(String teamID, String name2, int retval)
			throws SQLException {
		PreparedStatement ps = connection
				.prepareStatement("select uid from states where status_team=? and status_service=?");
		ps.setString(1, teamID);
		ps.setString(2, name2);
		ResultSet rs = ps.executeQuery();
		if (rs.next()) {
			ps = connection
					.prepareStatement("update states set status_text=? where uid=?");
			ps.setLong(2, rs.getLong(1));
			switch (retval) {
			case 0:
				ps.setString(1, "running");
				break;
			case 2:
				ps.setString(1, "not running");
				break;
			case 1:
				ps.setString(1, "broken");
				break;
			case 3:
				ps.setString(1, "timeout");
				break;
			}
			ps.execute();
			ps = connection
					.prepareStatement("update states set status_updated=? where uid=?");
			ps.setLong(2, rs.getLong(1));
			ps.setLong(1, Calendar.getInstance().getTimeInMillis() / 1000);
			ps.execute();
		} else {
			ps = connection
					.prepareStatement("insert into states (status_team,status_service,status_text,status_updated) values (?,?,?,?)");
			ps.setString(1, teamID);
			ps.setString(2, name2);
			switch (retval) {
			case 0:
				ps.setString(3, "running");
				break;
			case 2:
				ps.setString(3, "not running");
				break;
			case 1:
				ps.setString(3, "broken");
				break;
			case 3:
				ps.setString(3, "timeout");
				break;
			}
			ps.setLong(4, Calendar.getInstance().getTimeInMillis() / 1000);
			ps.execute();
		}
	}
}
